function [classes, outputs] = mperceptron_test(X, perceptron)
%% function [classes, outputs] = mperceptron_test(X, perceptron)
%%
%% Computes the classes predicted by a multiclass perceptron, together with
%% the values of $<w,X>$ for each text point
%%
%% INPUT
%% -----
%% X: test data, one example per row
%% perceptron: a perceptron structure cf. mperceptron_train
%%
%% OUTPUT
%% ------
%% classes: the classes that are predicted 
%% outputs: the outputs values w,X
%%
%% Author: L. Ralaivola
%% Date: Mar. 9, 2012
[n_data,dim] = size(X);

if(dim+1 ~= size(perceptron.w, 1))
    printf('problem in mperceptron_test: dimensions mismatch\n');
    exit(1);
end

X = [X, ones(n_data,1)];

outputs = X * perceptron.W;

[maxval, cl] = max(outputs');
classes = cl';
end