function [perceptron, updates] = mperceptron_train(X, Y, max_update)
%% function [perceptron, updates] = perceptron_train(X, Y, max_update)
%% function that learns a multiclass perceptron (max_update
%% procedure) from X and Y
%%
%% INPUT
%% -----
%% X: training data, one vector per row
%% Y: training labels, a column vector
%%
%% OUTPUT
%% ------
%% a perceptron making classification according to sign(wx + b)
%% perceptron.W: the classification vectors
%% updates: the number of updates made during the process
%%
%% Author: L. Ralaivola
%% Date: Mar. 9, 2012

[n_data, dim] = size(X);

if (size(Y,1) ~= n_data)
    printf(['error in mperceptron_train: number of labels and data ' ...
            'do not match\n']);
    exit(1);
end

if (nargin==2)
    max_update = Inf;
end

Q = max(Y);
X = [X, ones(n_data,1)];
W = zeros(dim+1, Q);

updates = 0;
margins = X * W;
[maxval, classes] = max(margins');
errors = find(Y ~= classes');

while (size(errors, 1)>0 && updates<max_update)
    updates = updates + 1;
    
    idx = errors(1);
    W(:, classes(idx)) = W(:,classes(idx)) - X(idx,:)';
    W(:, Y(idx)) = W(:, Y(idx)) + X(idx, :)';

    margins = X * W;
    [maxval, classes] = max(margins');
    errors = find(Y ~= classes');
end

perceptron.W = W;

end
