function [perceptron, updates] = perceptron_train(X, Y, max_update)
%% function [perceptron, updates] = perceptron_train(X, Y, max_update)
%% function that learns a perceptron from X and Y
%%
%% INPUT
%% -----
%% X: training data, one vector per row
%% Y: training labels, a column vector
%%
%% OUTPUT
%% ------
%% a perceptron making classification according to sign(wx + b)
%% perceptron.w: the classification vector
%% perceptron.b: the threshold
%% updates: the number of updates made during the process
%%
%% Author: L. Ralaivola
%% Date: Mar. 9, 2012

[n_data, dim] = size(X);

if (size(Y,1) ~= n_data)
    printf(['error in perceptron_train: number of labels and data ' ...
            'do not match\n']);
    exit(1);
end

if (nargin==2)
    max_update = Inf;
end
    
X = [X, ones(n_data,1)];
w = zeros(dim+1, 1);

updates = 0;
margins = Y .* (X * w);
errors = find(margins <= 0);

while (size(errors, 1)>0 && updates<max_update)
    updates = updates + 1;
    
    idx = errors(1);
    w = w + Y(idx) * X(idx,:)';
    
    margins = Y .* (X * w);
    errors = find(margins <= 0);
end

perceptron.w = w(1:dim,1);
perceptron.b = w(dim+1);

end
